
package App::Asciio::stripes::wirl_arrow ;

use parent qw/App::Asciio::stripes::stripes/ ;

use strict;
use warnings;


sub display_arrow_edit_dialog
{
my ($self) = @_ ;
}

#-----------------------------------------------------------------------------

1 ;









