
DRAW_CONNECTION_POINTS => 1,
DRAW_HINT_LINES => 0,
ACTION_VERBOSE => 0,
LAST_ACTION => '',
DIALOGS => 
	{
	#BOX_EDIT => '',
	},

TAB_AS_SPACES => '    ',

# use ANSI colors
COLOR_SCHEMES =>
	{
	'night' =>
		{
		background => 'on_black',
		grid => 'dark white',
		grid_2 => 'dark white',
		ruler_line => 'ansi88',
		hint_line => 'dark blue',
		hint_line2 => 'blue',
		selected_element_background => 'on_ansi31',
		element_background => 'on_black',
		element_foreground => 'white' ,
		group_colors =>
			[
			'on_ansi94',
			'on_ansi65',
			'on_ansi25',
			'on_ansi97',
			],
		connection => 'red',
		connection_point => 'ansi31',
		connector_point => 'green',
		extra_point => 'ansi31',
		new_connection => 'red',
		mouse_rectangle => 'red',
		selection_rectangle => 'ansi31',
		}, 
	'system' =>
		{
		background => 'on_black',
		grid => 'dark white',
		grid_2 => 'dark white',
		ruler_line => 'ansi88',
		hint_line => 'dark blue',
		hint_line2 => 'blue',
		selected_element_background => 'on_ansi31',
		element_background => 'on_black',
		element_foreground => 'white' ,
		group_colors =>
			[
			'on_ansi94',
			'on_ansi65',
			'on_ansi25',
			'on_ansi97',
			],
		connection => 'red',
		connection_point => 'ansi31',
		connector_point => 'green',
		extra_point => 'ansi31',
		new_connection => 'red',
		mouse_rectangle => 'red',
		selection_rectangle => 'ansi31',
		}, 
	},
